#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "mpi.h" 

int main(argc,argv)
int argc;
char *argv[];
{
    MPI_Comm icomm;
    int   nx, ny, nnx, nny, *id, *ik, nnode, sksize, ScaleUp,
      i, zero, procx, procy, numprocs, myid, NXglob, NYglob,
      neigh[4], color, ierr;
    double xl, xr, yb, yt, *sk, dzero=0., *u, *f, 
      omega,  done = 1.;
    
 
    icomm = MPI_COMM_WORLD;
    MPI_Init(&argc,&argv);
    MPI_Comm_size(icomm,&numprocs);
    MPI_Comm_rank(icomm,&myid);
    if (myid == 0)
    {
       printf("\n There are %d processes running.\n \n",numprocs);
    }
    
    MPI_Barrier(icomm);

    procx = (int) sqrt((double) numprocs);
    procy = (int) sqrt((double) numprocs);

    if (procy * procx != numprocs) 
    {
      printf("\n Wrong number of processors !\n \n");
    }
    else
    {
      IniGeom(myid, procx, procy, neigh, &color);

      MPI_Barrier(icomm);
      /*                                              subdomain coordinates */
      IniCoord(myid, procx, procy, &xl, &xr, &yb, &yt);

      ScaleUp = -1;
      /* number of local intervals on (xl,xr)=:nx, (yb,yt)=:ny */
      if (ScaleUp==1)
      {
      /*		for scale up evaluation   */ 
        if (myid==0) printf("scale up evaluation\n");
	nx = 10;
	ny = 10;
	NXglob = nx*procx;
	NYglob = ny*procy;
      }
      else
      {
      /*		for speed up evaluation   */ 
        if (myid==0) printf("speed up evaluation\n");
	NXglob = NYglob = 120; 
	nx = NXglob/procx;
	ny = NYglob/procy;
      }
      
      /* number of nodes per direction (xl,xr)=nx+1=:nex, (yb,yt)=ny+1=:ney */
      nnx = nx + 1;
      nny = ny + 1;
      /* number of degrees of freedom  */
      nnode = nnx*nny;
      
      /* allocate memory */
      u = (double *) malloc(nnode*sizeof(dzero));          /* temperature       */
      f = (double *) malloc(nnode*sizeof(dzero));          /* r.h.s.            */

      sksize = 4*nx*ny+2*(nx+ny)+1;			/* symm. storage     */
      sk = (double *) malloc(sksize*sizeof(dzero));	/* K                 */
      ik = (int *) malloc(sksize*sizeof(zero));		/* crs pointer for K */
      id = (int *) malloc(nnode*sizeof(zero));		/* crs pointer for K */

      /* matrix, rhs generation including Dirichlet B.C. */
      
      GetMatrix(nx, ny, xl, xr, yb, yt, sk, id, ik, f); /* generate K, rhs   */

      SetU(nx, ny, u);			     /* set solution/boundary values */
      ApplyDirichletBC(nx, ny, neigh, u, sk, id, ik, f); /* apply B.C.       */


      JacobiSolve(nx, ny, neigh, color, myid, icomm, sk, id, ik, f, u );

      SaveVectorP(myid, "t.dat", u, nx, ny, xl, xr, yb, yt, &ierr);
      
      MPI_Finalize();exit(0);

      
	/*  simple check of CrsMult function	*/ 
     	/* check matrix, rhs */      
      DebugMatrix(nnode, sk, id, ik); 
      DebugD(nnode, f);

      for (i=0; i<nnode; i++)
      	   u[i] = i;

      CrsMult(0, nnode-1, f, u, id, ik, sk, dzero);
      DebugD(nnode,f);
  
	/*  simple check of VecAccu function	*/ 
	/* initialize u  */
      for (i=0; i<nnode; i++)
      	   u[i] = myid*100+i;

      DebugD(nnode,u);
      
      VecAccu(nx,ny,u,neigh,color,myid,icomm);
      
      DebugD(nnode,u);
 
      
      free(id); free(ik); free(sk); free(f); free(u);
  }
	       
  MPI_Finalize();
}

            
